@@ -19,8 +19,9 @@
 #    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 #    MA 02111-1307  USA
 
-import unittest, pkg_resources
+import unittest
 
+import pkg_resources
 from dlg.dropmake.lg import LG
 from dlg.dropmake.pgt import PGT, GPGTNoNeedMergeException
 from dlg.dropmake.pgtp import MetisPGTP, MySarkarPGTP, MinNumPartsPGTP
@@ -58,14 +59,17 @@ class TestPGGen(unittest.TestCase):
         # pprint.pprint(dict(lg._drop_dict))
 
     def test_pgt_to_json(self):
-        fp = get_lg_fname("cont_img.graph")
+        fp = get_lg_fname("HelloWorld_simple.graph")
         lg = LG(fp)
         drop_list = lg.unroll_to_tpl()
         pgt = PGT(drop_list)
-        # print pgt.to_gojs_json()
+        pg_json = pgt.to_gojs_json()
+        _dum = pg_json
+        # we should really check the output here
 
     def test_metis_pgtp(self):
         lgnames = [
+            "HelloWorld_simple.graph",
             "simpleMKN.graph",
             "testLoop.graph",
             "cont_img.graph",
@@ -82,6 +86,7 @@ class TestPGGen(unittest.TestCase):
 
     def test_metis_pgtp_gen_pg(self):
         lgnames = [
+            "HelloWorld_simple.graph",
             "testLoop.graph",
             "cont_img.graph",
             "test_grpby_gather.graph",
@@ -232,5 +237,5 @@ class TestPGGen(unittest.TestCase):
             lg = LG(fp)
             out = lg.unroll_to_tpl()
             for drop in out:
-                if drop['type'] == 'plain':
-                    self.assertEqual("SharedMemory", drop['storage'])
+                if drop["type"] == "plain":
+                    self.assertEqual("SharedMemory", drop["storage"])
